' ------------------------------------------------------------------------------
' ILCtrl 1.05 for Windows
' Copyright (c) 2009-2019  J-F Garnier
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
' 
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' ----------------------------------------------------------------------------------
'
' MainForm.vb  Main module, GUI, PIL-Box management
'
' Nov  2009: Version 1.0 Demonstrator for PILBox usage as a HPIL controller
' Dec  2009: Version 1.01 Minor fix
' May  2014: Version 1.02 Fix problem in case of no response to SOT
'                         Ensure to restart with correct high byte for PIL-Box
'                         SPOLL can return up to 4 bytes
'                         Scope can monitor either output or input frames
'                         disable the command buttons during the timeout delays
' July 2016: Version 1.03 Add the 230 kbps setting
' Apr  2017: Version 1.04 Fix issues with the HP3468 (no IFC during transaction, 256 byte max)
' Jan  2019: Version 1.05 Fix scope display (design), limit addr to 31, add NO RESPONSE message
' ---------------------------------------------------------------------------------


' --------------------------------
' MainForm
' Main application form for ILCtrl
' --------------------------------
Public Class MainForm

    Dim lasth As Integer = 0      ' store the last sent/received high byte on serial
    Dim frame_timeout As Boolean

   ' PIL-Box special init command frames:
    Const CON = &H496     ' CON
    Const COFF = &H497    ' COFF
    Const TDIS = &H494    ' TDIS

    Const USE8BITS = True ' use 8-bit format with PILBox

    Const CRLF = Chr(13) & Chr(10)  ' convenient string

    ' --------------------------------
    ' DisplayStr()
    ' display a string in the display box
    ' --------------------------------
    Public Sub DisplayStr(ByVal ch As String)
        TextBox_display.AppendText(ch)
    End Sub

    ' --------------------------------
    ' ClrDisplay()
    ' clear the display box
    ' --------------------------------
    Public Sub ClrDisplay()
        TextBox_display.Clear()
    End Sub

    ' --------------------------------
    ' DisplayMnemo()
    ' display a string in the scope box
    ' --------------------------------
    Public Sub DisplayMnemo(ByVal ch As String)
        Static cpt As Integer = 0
        TextBox_scope.AppendText(ch)
        cpt = cpt + 1
        If cpt = 3 Then
            cpt = 0
            TextBox_scope.AppendText(CRLF)
        End If
    End Sub

    ' --------------------------------
    ' v1.02
    ' DisplayMnemo_out()
    ' display a string in the scope box
    ' (if the scope is enabled for output frames)
    ' --------------------------------
    Public Sub DisplayMnemo_out(ByVal ch As String)
        If CheckBox_scope.Checked And RadioButton_out.Checked Then
            DisplayMnemo(ch)
        End If
    End Sub

    ' --------------------------------
    ' v1.02
    ' DisplayMnemo_in()
    ' display a string in the scope box
    ' (if the scope is enabled for input frames)
    ' --------------------------------
    Public Sub DisplayMnemo_in(ByVal ch As String)
        If CheckBox_scope.Checked And RadioButton_in.Checked Then
            DisplayMnemo(ch)
        End If
    End Sub


    ' --------------------------------
    ' START button 
    ' --------------------------------
    Private Sub ButtonStart_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ButtonStart.Click

        ClrDisplay()

        SerialPort1.PortName = TextBox_ComPort.Text
        If RadioButton_highspeed.Checked Then SerialPort1.BaudRate = 230400 Else SerialPort1.BaudRate = 115200

        Try
            SerialPort1.Open()
        Catch
            MessageBox.Show("Error: COM port not available.", "My Application", _
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End Try

        ' init the PIL-Box in control-on mode 
        If InitPILBox(CON) < 0 Then
            MessageBox.Show("Error: no response from PILBox.", "My Application", _
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            SerialPort1.Close()
            Exit Sub
        End If

        ButtonStop.Enabled = True
        ButtonStart.Enabled = False
        GroupBox_Ctrl.Enabled = True
        TextBox_ComPort.Enabled = False
        RadioButton_lowspeed.Enabled = False
        RadioButton_highspeed.Enabled = False

    End Sub

    Private Sub StopPILbox()
        Try
            InitPILBox(TDIS)   ' stop the PIL-Box       
        Catch
            ' in case the PILBox is disconnected before closing ILctrl...
        End Try
        Try
            SerialPort1.Close()
        Catch
            ' in case the PILBox is disconnected before closing ILctrl...
        End Try

    End Sub


    ' --------------------------------
    ' STOP button
    ' --------------------------------
    Private Sub ButtonStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStop.Click
        ButtonStop.Enabled = False
        ButtonStart.Enabled = True
        GroupBox_Ctrl.Enabled = False
        TextBox_ComPort.Enabled = True
        RadioButton_lowspeed.Enabled = True
        RadioButton_highspeed.Enabled = True
        StopPILbox()
    End Sub


    ' --------------------------------
    ' Form closing
    ' --------------------------------
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        StopPILbox()
    End Sub



    ' --------------------------------
    ' Command buttons
    ' --------------------------------

    Private Sub Button_send_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_send.Click
        Dim n As Integer
        Dim s As String

        n = TextBox_addr.Text
        If n > 31 Then n = 31 ' v1.05
        s = TextBox_data.Text & CRLF
        DisplayStr("SEND (" & n & "): " & s)
        GroupBox_Ctrl.Enabled = False
        SendData(n, s)
        GroupBox_Ctrl.Enabled = True
    End Sub

    Private Sub Button_enter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_enter.Click
        Dim n As Integer
        Dim s As String

        n = TextBox_addr.Text
        If n > 31 Then n = 31 ' v1.05
        DisplayStr("ENTER (" & n & "): ")
        s = ""
        GroupBox_Ctrl.Enabled = False
        GetData(n, SDA, s)
        GroupBox_Ctrl.Enabled = True
        If Microsoft.VisualBasic.Right(s, 2) <> CRLF Then s = s & CRLF
        DisplayStr(s)
    End Sub

    Private Sub Button_did_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_did.Click
        Dim n As Integer
        Dim s As String

        n = TextBox_addr.Text
        If n > 31 Then n = 31 ' v1.05
        DisplayStr("Device ID (" & n & "): ")
        s = ""
        GroupBox_Ctrl.Enabled = False
        GetData(n, SDI, s)
        GroupBox_Ctrl.Enabled = True
        If Microsoft.VisualBasic.Right(s, 2) <> CRLF Then s = s & CRLF
        DisplayStr(s)
    End Sub

    Private Sub Button_remote_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_remote.Click
        DisplayStr("REMOTE" & CRLF)
        SendUCG(REN)
    End Sub

    Private Sub Button_local_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_local.Click
        DisplayStr("LOCAL" & CRLF)
        SendUCG(NRE)
    End Sub

    Private Sub Button_clear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_clear.Click
        DisplayStr("CLEAR" & CRLF)
        SendUCG(DCL)
    End Sub

    Private Sub Button_spoll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_spoll.Click
        Dim n As Integer, i As Integer
        Dim b(2) As Byte

        n = TextBox_addr.Text
        If n > 31 Then n = 31 ' v1.05
        DisplayStr("SPOLL(" & n & "): ")
        ' v1.02: up to 4 bytes
        GroupBox_Ctrl.Enabled = False
        n = GetBytes(n, SST, b, 4)
        GroupBox_Ctrl.Enabled = True
        If (n > 0) Then
            For i = 1 To n
                DisplayStr(b(i - 1))
                DisplayStr(" ")
            Next i
        End If
        DisplayStr(CRLF)
    End Sub

    Private Sub Button_trigger_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_trigger.Click
        Dim n As Integer

        n = TextBox_addr.Text
        If n > 31 Then n = 31 ' v1.05
        DisplayStr("TRIGGER " & n & CRLF)
        SendACG(n, _GET)
    End Sub




    ' ------------------------------------------------------------
    ' HP-IL primitives
    ' ------------------------------------------------------------

    ' HP-IL frame definition
    ' command frames
    Const _GET = &H408 '  (GET is a VB reserved word)
    Const DCL = &H414
    Const LAD = &H420
    Const UNL = &H43F
    Const TAD = &H440
    Const UNT = &H45F
    Const IFC = &H490
    Const REN = &H492
    Const NRE = &H493
    Const AAU = &H49A
    ' ready frames
    Const ETO = &H540
    Const ETE = &H541
    Const NRD = &H542
    Const SDA = &H560
    Const SST = &H561
    Const SDI = &H562
    Const AAD = &H580


    ' starts a HPIL transaction (IFC, AAU, AAD 1)
    Private Function StartTrans(doifc As Integer) ' v1.04, add doifc condition
        lasth = 0   ' force the sending of the high byte to make sure PIL-Box com is synchr (v1.02)
        If (doifc > 0) Then
            If SendFrame(IFC) <> IFC Then GoTo err
        End If
        If SendFrame(AAU) <> AAU Then GoTo err
        If SendFrame(AAD + 1) < 0 Then GoTo err
        Return 0
err:    Return -1
    End Function

    ' ends a HPIL transaction (UNL, UNT)
    Private Sub EndTrans()
        lasth = 0   ' force the sending of the high byte to make sure PIL-Box com is synchr (v1.02)
        SendFrame(UNL)
        SendFrame(UNT)
    End Sub


    ' send an Universal Command Group frame
    Private Sub SendUCG(ByVal frame As Integer)
        If StartTrans(1) < 0 Then GoTo err
        SendFrame(frame)
        Exit Sub
err:
        DisplayStr("TRANSMIT ERR" & CRLF)
    End Sub

    ' send an Addressed Command Group frame
    Private Sub SendACG(ByVal addr As Integer, ByVal frame As Integer)
        If StartTrans(0) < 0 Then GoTo err
        SendFrame(LAD + addr) ' address device as listener
        SendFrame(frame)
        EndTrans()
        Exit Sub
err:
        DisplayStr("TRANSMIT ERR" & CRLF)
    End Sub


    ' send a data string to device at addr 
    Private Sub SendData(ByVal addr As Integer, ByVal s As String)
        Dim i As Integer

        If StartTrans(0) < 0 Then GoTo err
        SendFrame(LAD + addr) ' address device as listener
        For i = 1 To Len(s)
            If (SendFrame(Asc(Mid(s, i))) < 0) Then GoTo err  ' v1.02
        Next i
        EndTrans()
        Exit Sub
err:
        DisplayStr("TRANSMIT ERR" & CRLF)
    End Sub


    ' get a data string from device at addr n with sot frame SDA or SDI
    ' end the transmission if ETO/ETE received,
    ' interrupts the transfer if LF received,  END byte received or 256 bytes received (v1.04: 256 bytes max)
    Private Function GetData(ByVal addr As Integer, ByVal sot As Integer, ByRef s As String) As Integer
        Dim frame As Integer
        Dim i As Integer

        s = "" : i = 0
        If StartTrans(0) < 0 Then GoTo err
        SendFrame(TAD + addr) ' address device as talker
        frame = SendFrame(sot) ' send start of transmission frame (SDI or SDA)
        If (frame < 0) Then GoTo err ' no response to sot (v1.02)
        ' if sot frame returns, it means the addressed device didn't answer (this is not an error)
        If (frame <> sot) Then
            Do
                If (frame = ETO) Then Exit Do ' end of transmission ok
                If (frame = ETE) Then Exit Do ' end of transmission error
                If (frame < &H400) Then
                    ' data or end frames
                    s = s & Chr(frame And &HFF) ' append to string
                    i = i + 1
                    If ((frame And &HFF) = 10) Or ((frame And &H200) = &H200) Or (i > 256) Then
                        ' LF received or END byte received or 256 bytes received (v1.04: 256 bytes max),
                        ' insert a  Not Ready for Data  frame to interrupt transfer
                        If SendFrame(NRD) <> NRD Then GoTo err
                    End If
                End If
                frame = SendFrame(frame) ' retransmit frame and get next one
                If frame < 0 Then Exit Do  ' v1.02
            Loop
        End If
        EndTrans()
        If (frame = ETE) Or (frame < 0) Then GoTo err
        If (frame = sot) Then
            DisplayStr("NO RESPONSE")   ' v1.05
        End If
        Return i
        Exit Function
err:
        DisplayStr("TRANSMIT ERR" & CRLF)
        Return -1
    End Function


    ' get binary data from device at addr with sot frame SDA, SAI or SST
    ' end the transmission if ETO/ETE received,
    ' interrupts the transfer after n bytes received
    Private Function GetBytes(ByVal addr As Integer, ByVal sot As Integer, ByRef s() As Byte, ByVal n As Integer) As Integer
        Dim frame As Integer
        Dim i As Integer

        If StartTrans(0) < 0 Then GoTo err
        SendFrame(TAD + addr) ' address device n as talker
        i = 0
        frame = SendFrame(sot) ' send start of transmission frame (SAI, SST or SDA)
        If (frame < 0) Then GoTo err ' no response to sot (v1.02)
        ' if sot frame returns, it means the addressed device didn't answer (this is not an error)
        If (frame <> sot) Then
            Do
                If (frame = ETO) Then Exit Do ' end of transmission ok
                If (frame = ETE) Then Exit Do ' end of transmission error
                If (frame < &H400) Then
                    ' data or end frames
                    s(i) = frame And &HFF
                    i = i + 1
                    If (i >= n) Then
                        ' insert a  Not Ready for Data  frame to interrupt transfer
                        If SendFrame(NRD) <> NRD Then GoTo err
                    End If
                End If
                frame = SendFrame(frame) ' retransmit frame and get next one
                If frame < 0 Then Exit Do ' v1.02
            Loop
        End If
        EndTrans()
        If (frame = ETE) Or (frame < 0) Then GoTo err
        If (frame = sot) Then
            DisplayStr("NO RESPONSE")   ' v1.05
        End If
        Return i
        Exit Function
err:
        DisplayStr("TRANSMIT ERR" & CRLF)
        Return -1
    End Function




    ' ------------------------------------------------------------
    ' PILBox functions
    ' ------------------------------------------------------------

    ' --------------------------------
    ' InitPILBox()
    ' init the PIL Box 
    ' --------------------------------
    Private Function InitPILBox(ByVal cmd As Integer) As Integer

        lasth = 0   ' force the sending of the high byte to make sure PIL-Box com is synchr
        If SendFrame(cmd) <> cmd Then
            Return -1  ' error
        Else
            Return 0
        End If

    End Function


    ' --------------------------------
    ' send a frame to the PILBox, wait for the returned frame
    ' frame<0 means no frame returns (timeout)
    ' --------------------------------
    Private Function SendFrame(ByVal frame As Integer) As Integer
        Dim lbyt As Integer, hbyt As Integer  ' low/high bytes to serial
        Dim buf(2) As Byte, byt As Byte
        Dim n As Integer
        Dim s As String

        If (frame < 0) Then GoTo Err ' v1.02

        ' build the low and high parts
        If Not USE8BITS Then
            ' use 7-bit characters for maximum compatibility
            hbyt = ((frame >> 6) And &H1F) Or &H20
            lbyt = (frame And &H3F) Or &H40
        Else
            ' 8-bit format for optimum speed
            hbyt = ((frame >> 6) And &H1E) Or &H20
            lbyt = (frame And &H7F) Or &H80
        End If
        If (hbyt <> lasth) Then
            ' send high part if different from last one
            lasth = hbyt
            buf(1) = hbyt : buf(2) = lbyt : n = 2
        Else
            ' otherwise send only low part
            buf(1) = lbyt : n = 1
        End If
        SerialPort1.DiscardInBuffer() ' added v 1.01 in case of timeout
        SerialPort1.Write(buf, 1, n)

        ' scope out - before frame received
        s = ILMnemo(frame)
        DisplayMnemo_out(s)

        ' wait for returned frame
        ' timeouts: 10s for DOE/RDY, 3s for CMD, 1s for IDY 
        If frame < &H400 Then
            Timer1.Interval = 10000 ' DOE
        ElseIf (frame And &H7F4) = &H494 Then
            Timer1.Interval = 500  ' special PILBox frames 494-497 and 49c-49f
        ElseIf frame < &H500 Then
            Timer1.Interval = 3000  ' CMD
        ElseIf frame < &H600 Then
            Timer1.Interval = 10000  ' RDY
        Else
            Timer1.Interval = 1000  ' IDY
        End If
        frame = -1
        frame_timeout = False
        Timer1.Enabled = True

        Do

            Try
                byt = SerialPort1.ReadByte()
            Catch
                byt = 0
            End Try
            If byt <> 0 Then
                If ((byt And &HC0) = 0) Then
                    If (byt And &H20) Then
                        ' high byte, save it
                        lasth = byt
                        ' SerialPort1.Write(Chr(13))   ' acknowledge no more needed (v1.03)
                    End If
                Else
                    ' low byte, build frame according to format 
                    If (byt And &H80) Then
                        frame = ((lasth And &H1E) << 6) + (byt And &H7F)
                    Else
                        frame = ((lasth And &H1F) << 6) + (byt And &H3F)
                    End If
                End If
            End If

            My.Application.DoEvents()

        Loop Until frame_timeout Or (frame >= 0)

        If (frame >= 0) Then
            ' scope in - after frame received
            s = ILMnemo(frame)
            DisplayMnemo_in(s)
        End If

        Timer1.Enabled = False
Err:    Return frame

    End Function


    ' handler for receive timeout
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        frame_timeout = True
    End Sub


End Class
